'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buildTypes = exports.ensureNoDuplicateTypes = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _buildMap;

exports.default = build;

var _type = require('graphql/type');

var _language = require('graphql/language');

var _buildSchema = require('./buildSchema');

var _buildSchema2 = _interopRequireDefault(_buildSchema);

var _buildScalar = require('./buildScalar');

var _buildScalar2 = _interopRequireDefault(_buildScalar);

var _buildObject = require('./buildObject');

var _buildObject2 = _interopRequireDefault(_buildObject);

var _buildInterface = require('./buildInterface');

var _buildInterface2 = _interopRequireDefault(_buildInterface);

var _buildUnion = require('./buildUnion');

var _buildUnion2 = _interopRequireDefault(_buildUnion);

var _buildEnum = require('./buildEnum');

var _buildEnum2 = _interopRequireDefault(_buildEnum);

var _buildInputObject = require('./buildInputObject');

var _buildInputObject2 = _interopRequireDefault(_buildInputObject);

var _resolveThunk = require('./resolveThunk');

var _resolveThunk2 = _interopRequireDefault(_resolveThunk);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var buildMap = (_buildMap = {}, _defineProperty(_buildMap, _language.Kind.SCALAR_TYPE_DEFINITION, _buildScalar2.default), _defineProperty(_buildMap, _language.Kind.OBJECT_TYPE_DEFINITION, _buildObject2.default), _defineProperty(_buildMap, _language.Kind.INTERFACE_TYPE_DEFINITION, _buildInterface2.default), _defineProperty(_buildMap, _language.Kind.UNION_TYPE_DEFINITION, _buildUnion2.default), _defineProperty(_buildMap, _language.Kind.ENUM_TYPE_DEFINITION, _buildEnum2.default), _defineProperty(_buildMap, _language.Kind.INPUT_OBJECT_TYPE_DEFINITION, _buildInputObject2.default), _buildMap);

var ensureNoDuplicateTypes = exports.ensureNoDuplicateTypes = function ensureNoDuplicateTypes(types) {
  types.forEach(function (typeA) {
    if (types.some(function (typeB) {
      return typeA !== typeB && typeA.name === typeB.name;
    })) {
      throw new Error('Duplicate type "' + typeA.name + '" in schema definition or type dependencies.');
    }
  });
  return types;
};

var buildTypes = exports.buildTypes = function buildTypes(documentAST) {
  var configMap = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var typeDependencies = arguments.length <= 2 || arguments[2] === undefined ? [] : arguments[2];

  var derived = documentAST.definitions.map(function (definition) {
    var buildDefinition = buildMap[definition.kind];
    if (!buildDefinition) return false;
    return buildDefinition(definition, configMap[definition.name.value], function () {
      return ensureNoDuplicateTypes([].concat(_toConsumableArray(derived), _toConsumableArray((0, _resolveThunk2.default)(typeDependencies))));
    });
  }).filter(function (x) {
    return x;
  });
  return derived;
};

function build(source) {
  var config = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var typeDeps = arguments.length <= 2 || arguments[2] === undefined ? [] : arguments[2];
  var infer = arguments.length <= 3 || arguments[3] === undefined ? true : arguments[3];

  if (typeof source !== 'string') throw new Error('Expected a string but got ' + source + '.');

  /* eslint-disable no-param-reassign */
  if (Array.isArray(config) || typeof config === 'function') {
    infer = typeof typeDeps === 'boolean' ? typeDeps : true;
    typeDeps = config;
    config = {};
  }

  if (typeof config === 'boolean') {
    infer = config;
    typeDeps = [];
    config = {};
  }

  if (typeof typeDeps === 'boolean') {
    infer = typeDeps;
    typeDeps = [];
  }
  /* eslint-enable no-param-reassign */

  if (!(config instanceof Object)) throw new Error('Expected an object but got ' + config + '.');
  if (!(Array.isArray(typeDeps) || typeof typeDeps === 'function')) {
    throw new Error('Expected an array or a function but got ' + typeDeps + '.');
  }
  if (typeof infer !== 'boolean') throw new Error('Expected a boolean but got ' + infer + '.');

  var documentAST = (0, _language.parse)(source);
  var definitions = documentAST.definitions;

  var firstDefinition = definitions[0];
  var shouldReturnOneType = definitions.length === 1 && ~Object.keys(buildMap).indexOf(firstDefinition.kind) && !(firstDefinition.name.value === 'Query' && infer);
  var types = ensureNoDuplicateTypes(buildTypes(documentAST, shouldReturnOneType ? _defineProperty({}, firstDefinition.name.value, config) : config, typeDeps));

  if (shouldReturnOneType) return types[0];

  // try to build the defined schema
  var schemaASTs = definitions.filter(function (_ref2) {
    var kind = _ref2.kind;
    return kind === _language.Kind.SCHEMA_DEFINITION;
  });
  if (schemaASTs.length > 1) throw new Error('Must provide only one schema definition.');
  if (schemaASTs.length === 1) {
    if (!Array.isArray(typeDeps)) {
      throw new Error('Can\'t use thunks as type dependencies for schema.');
    }
    return (0, _buildSchema2.default)(schemaASTs[0], undefined, function () {
      return ensureNoDuplicateTypes([].concat(_toConsumableArray(types), _toConsumableArray(typeDeps)));
    });
  }

  // try to build an inferred schema
  if (infer) {
    var QueryType = types.find(function (_ref3) {
      var name = _ref3.name;
      return name === 'Query';
    });
    if (QueryType) {
      if (!Array.isArray(typeDeps)) {
        throw new Error('Can\'t use thunks as type dependencies for schema.');
      }
      var allTypes = ensureNoDuplicateTypes([].concat(_toConsumableArray(types), _toConsumableArray(typeDeps)));
      return new _type.GraphQLSchema({
        query: QueryType,
        mutation: allTypes.find(function (_ref4) {
          var name = _ref4.name;
          return name === 'Mutation';
        }),
        subscription: allTypes.find(function (_ref5) {
          var name = _ref5.name;
          return name === 'Subscription';
        })
      });
    }
  }

  // return a type map
  return types.reduce(function (map, type) {
    return _extends({}, map, _defineProperty({}, type.name, type));
  }, {});
}